@@ -1,21 +1,29 @@
 from dataclasses import dataclass, field, replace
+from typing import Any, Dict, Iterable, Iterator, NamedTuple, Sequence, Union
+
+
+@dataclass(frozen=True, order=True)
+class ChunkHeader:
+    tag: bytes
+    size: int
 
-from typing import NamedTuple, Iterable, Iterator, Sequence
 
 class Chunk(NamedTuple):
     """Chunk made of 4CC header and data
-    
+
     tag: 4CC header
 
     data: chunk data
     """
+
     tag: str
     data: bytes
 
+
 @dataclass
 class Element:
     """Indexing metadata for chunk containers
-    
+
     tag: 4CC header
 
     data: chunk data
@@ -24,9 +32,10 @@ class Element:
 
     children: Contained elements
     """
+
     tag: str
     data: bytes = field(repr=False)
-    attribs: dict
+    attribs: Dict[str, Any]
     children: Sequence['Element']
 
     def __iter__(self) -> Iterator['Element']:
@@ -38,3 +47,6 @@ class Element:
     def chunk(self) -> Chunk:
         """Convert element to chunk"""
         return Chunk(self.tag, self.data)
+
+
+ElementTree = Union[Iterator[Element], Element, None]
